<?php
namespace Teknisa\Model\Geral\Repositories;


use OrganizationsManagement\Util\Utilities;
use Doctrine\DBAL\Connection;


class Organizacao extends \Doctrine\ORM\EntityRepository {

    public function retornaMaxOrganizacao()
    {
        $sqlMax = "SELECT MAX(NRORG)+1 FROM ORGANIZACAO";

        $result = $this->getEntityManager()->getConnection()->fetchColumn($sqlMax);
        return ($result) ? $result : [];
    }

    public function obtemTodasOrganizacoes($conditions, $limit, $offset){
        $query = <<<SQL
        SELECT *
          FROM ( SELECT O.NRORG, O.NMRAZSOCIALORG, O.NMORGANIZACAO, O.PREPSTATUS, O.IDATIVO,
                        O.IDTIPOINSCRORG, O.NRORGPRINCIPAL, O.PREPSTATUS AS ORGSTATUS,
                        DECODE(O.NRORGPRINCIPAL, NULL, 'P', 'S') AS IS_PRIMARY,
                        DECODE(O.NRORGPRINCIPAL, NULL, O.IDORGSAAS, OM.IDORGSAAS) AS IDORGSAAS
                   FROM ORGANIZACAO O
              LEFT JOIN ORGANIZACAO OM ON (O.NRORGPRINCIPAL = OM.NRORG))
         WHERE 1 = 1
              $conditions
SQL;

        $query  = Utilities::getPaginatedQuery($query, $limit, $offset);
        $result = $this->getEntityManager()->getConnection()->fetchAll($query);
        return ($result) ? $result : [];
    }

    public function allOrganizationRelations($conditions, $nrorg) {
        $query = <<<SQL
        SELECT *
          FROM ( SELECT NRORG, NMRAZSOCIALORG, NMORGANIZACAO, PREPSTATUS, IDATIVO, IDTIPOINSCRORG, 
                        IDORGSAAS, NRORGPRINCIPAL, PREPSTATUS AS ORGSTATUS
                   FROM ORGANIZACAO
                  WHERE NRORGPRINCIPAL = :nrorg
                    AND IDATIVO = 'S' )
         WHERE 1=1
              $conditions
SQL;
        $params = array(
            "nrorg" => $nrorg
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query,$params);
    }



    public function obtemTodasOrganizacoesComElemento($conditions, $limit, $offset){
        $query = <<<SQL
               SELECT DISTINCT ATIVO, NRORG, NMORGANIZACAO, NMRAZSOCIALORG, ORGSTATUS,
                              NMPRODUTO, ID, DSEMAILOPER, ORGID, IS_PRIMARY, IDORGSAAS, DSURLACESSO
                 FROM (SELECT DISTINCT OZ.IDATIVO AS ATIVO, PE.LABEL, OZ.NRORG, OZ.NMORGANIZACAO, OZ.NMRAZSOCIALORG,
                              OP.NMPRODUTO, OP.ID, OZ.DSEMAILOPER, OG.ID AS ORGID, OZ.PREPSTATUS AS ORGSTATUS, 
                              DECODE(OZ.NRORGPRINCIPAL, NULL, 'P', 'S') AS IS_PRIMARY,
                              DECODE(OZ.NRORGPRINCIPAL, NULL, OZ.IDORGSAAS, OM.IDORGSAAS) AS IDORGSAAS, OG.DSURLACESSO
                         FROM ORGANIZACAO OZ
                    LEFT JOIN OM_PRODUTO_ORG OG ON OG.NRORG_ID = OZ.NRORG
                    LEFT JOIN OM_PRODUTO OP ON OP.ID = OG.NRPRODUTO_ID
                    LEFT JOIN ORGANIZACAO OM ON OZ.NRORGPRINCIPAL = OM.NRORG
                    LEFT JOIN OM_PRODUTO_ORG_ELEMENTO EL ON OG.ID = EL.NRPRODUTO_ORG_ID
                    LEFT JOIN OM_PRODUTO_ELEMENTO PE ON EL.NRPRODUTO_ELEMENTO_ID = PE.ID
                       )
               WHERE 1 = 1
               $conditions

SQL;
        $query  = Utilities::getPaginatedQuery($query, $limit, $offset);
        $result = $this->getEntityManager()->getConnection()->fetchAll($query);
        return ($result) ? $result : [];
    }

    public function obtemOrganizacoesComProduto($conditions, $limit, $offset, $orderBy = ''){
        $query = <<<SQL
              SELECT NRORG, NMORGANIZACAO, NMRAZSOCIALORG
                 FROM (SELECT OZ.NRORG, OZ.NMORGANIZACAO, OZ.NMRAZSOCIALORG, OP.ID
                         FROM OM_PRODUTO OP
                         JOIN OM_PRODUTO_ORG OG ON OG.NRPRODUTO_ID = OP.ID
                         JOIN ORGANIZACAO OZ ON OZ.NRORG = OG.NRORG_ID
                       )
               WHERE 1 = 1
               $conditions
               GROUP BY NRORG, NMORGANIZACAO, NMRAZSOCIALORG
               $orderBy
SQL;
        $query  = Utilities::getPaginatedQuery($query, $limit, $offset);
        $result = $this->getEntityManager()->getConnection()->fetchAll($query);
        return ($result) ? $result : [];
    }

    public function obtemPorId($id){
        $query = "
        SELECT O.IDTIPOINSCRORG, O.NMRAZSOCIALORG, O.NMORGANIZACAO, O.DSDIRLOG,
               O.NRORG, O.IDATIVO, O.NMACESSO, O.CDPAIS, O.SGESTADO,
               O.CDMUNICIPIO, O.NMBAIREMPR AS SA_BAIRRO, O.NRCEPEMPR, O.NRINSJUREMPR,
               O.NRINSCESTA, O.DSLOGRADEMPR, O.NRLOGRADEMPR, O.DSCMPLOGEMPR,
               O.CDINSCMUNI, O.DSEMAILEMPRESA, O.NRTELEFOEMPR, O.NMOPERADOR,
               O.DSEMAILOPER, SP.NMPAIS AS SA_PAIS, SE.NMESTADO AS SA_ESTADO, O.PREPSTATUS, O.PREPSTATUSBKP,
               O.DTULTATU, O.NRINSCRICAOORG, O.IDTIPOINSCRORG AS TIPOORG,
               DECODE(O.NRORGPRINCIPAL, NULL, O.IDORGSAAS, OM.IDORGSAAS) AS IDORGSAAS,
               O.NRORGPRINCIPAL, SM.NMMUNICIPIO AS SA_MUNICIPIO, OP.QTD_OPERADORES, O.NRPARCNEGOCIO,
               DECODE(O.NRORGPRINCIPAL, NULL, '', (O.NRORGPRINCIPAL || ' - ' || OM.NMORGANIZACAO)) AS NMORGPRINCIPAL,
               DECODE(O.NRORGPRINCIPAL, NULL, 'P', 'S') AS IS_PRIMARY,
               CASE WHEN O.PREPSTATUS IN ('C', 'S', 'N', 'O')
                    THEN 'S'
                    ELSE 'N'
                END AS EDIT_IS_PRIMARY,
               CASE WHEN SEMP.NRORG IS NULL
                    THEN 'N'
                    ELSE 'S'
                END AS ESTA_PREPARADA
          FROM (SELECT COUNT(*) AS QTD_OPERADORES
                  FROM USR_ORG_SAAS.OPERADOR
                 WHERE NRORG = :id) OP,
               ORGANIZACAO O
          LEFT JOIN PAIS SP        ON (O.CDPAIS = SP.CDPAIS)
          LEFT JOIN ESTADO SE      ON (O.SGESTADO = SE.SGESTADO AND SE.CDPAIS = SP.CDPAIS)
          LEFT JOIN MUNICIPIO SM   ON (O.CDMUNICIPIO = SM.CDMUNICIPIO AND O.SGESTADO = SM.SGESTADO AND SM.CDPAIS = SP.CDPAIS)
          LEFT JOIN EMPRESA SEMP   ON (O.NRORG = SEMP.NRORG)

          LEFT JOIN ORGANIZACAO OM ON (O.NRORGPRINCIPAL = OM.NRORG)
         WHERE O.NRORG = :id
        ";
        $param = array("id" => $id);

        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        return ($result) ? $result : [];
    }

    public function executaQueryParaPrepararEstrutura($query, $entityManager = null){
        $entityManager = $entityManager ? $entityManager : $this->getEntityManager();
        $entityManager->getConnection()->executeQuery($query);
        return $query;
    }

    public function updateOrganizationPrepStatus($dbUser, $presStatus, $id) {
        $query = "
        UPDATE $dbUser.ORGANIZACAO
           SET PREPSTATUS = :PREPSTATUS
         WHERE NRORG = :NRORG
        ";
        $param = array("NRORG" => $id, "PREPSTATUS" => $presStatus);
        $this->getEntityManager()->getConnection()->executeQuery($query, $param);
    }

    public function getOrganizacaoByNrorg($nrOrg) {
        $query = "
            SELECT NRORG, NMORGANIZACAO, NRORGPRINCIPAL, IDORGSAAS
              FROM ORGANIZACAO
             WHERE NRORG = :NRORG
        ";
        $param  = array("NRORG" => $nrOrg );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        return empty($result) ? $result : $result[0];
    }

    public function getOrganizacaoByNrorgPrincipal($nrOrg) {
        $query = "            
            SELECT NRORG, NMORGANIZACAO, NRORGPRINCIPAL
              FROM ORGANIZACAO
             WHERE IDATIVO = 'S'
               AND (NRORG = :NRORGPRINCIPAL
                    OR NRORGPRINCIPAL = :NRORGPRINCIPAL)
        ";
        $param  = array("NRORGPRINCIPAL" => $nrOrg );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    /**
     * @param $productId
     * @param string $filters
     * @return array
     */
    public function findOrganizationInformation($productId, $filters = '')
    {
        $query = <<<SQL
            SELECT *
              FROM (SELECT DISTINCT OZ.NRORG, OZ.NMORGANIZACAO, OZ.NMRAZSOCIALORG, OZ.IDORGSAAS, OP.NMPRODUTO, OG.ID AS NRPRODUTO_ORG_ID,
                                    SE.ID AS SERVER_ID,  SE.NAME AS SERVER_NAME, PS.PATH AS SERVER_PATH
                               FROM OM_PRODUTO OP
                               JOIN OM_PRODUTO_ORG OG ON OG.NRPRODUTO_ID = OP.ID
                               JOIN ORGANIZACAO OZ    ON OZ.NRORG = OG.NRORG_ID
                               JOIN CLIENTE CL        ON CL.NRPARCNEGOCIO = OZ.NRPARCNEGOCIO
                              
                               JOIN PRODUCT PR        ON PR.ID = OP.NREXTERNAL_PRODUCT_ID
                               JOIN PRODUCT_SERVER PS ON PS.PRODUCT_ID = PR.ID
                               JOIN SERVER SE         ON SE.ID = PS.SERVER_ID
                               JOIN RESOURCE_T RT     ON RT.ID = SE.RESOURCE_T_ID AND RT.EXTERNAL_CODE = CL.CDCLIENTE
                              WHERE OP.NREXTERNAL_PRODUCT_ID = :PRODUCT_ID
                                AND OP.IDSTATUS = 'A'
                                AND OZ.IDATIVO = 'S'
                                AND OG.IDATIVO = 'S'
                                AND PR.IS_ACTIVE = 'Y'
                                AND PS.IS_ACTIVE = 'Y'
                                AND SE.IS_ACTIVE = 'Y'
                           ORDER BY OZ.NMORGANIZACAO)
             WHERE 1 = 1
             $filters
SQL;
        $param = array("PRODUCT_ID" => $productId);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function findRelatedOrganizations($orgIds)
    {
        $query = "SELECT NRORG, NMORGANIZACAO, NMRAZSOCIALORG, NMORGANIZACAO AS NMORGANIZACAOOPER,
                         NRINSJUREMPR, DECODE (NRORGPRINCIPAL, NULL, 'P', 'S') AS NRORGPRINCIPAL
                    FROM ORGANIZACAO
                   WHERE IDATIVO = 'S'
                     AND NRORG IN (:NRORGS)";
        $params = array(
            'NRORGS' => $orgIds
        );
        $paramsTypes = array(
            'NRORGS' => Connection::PARAM_STR_ARRAY
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramsTypes);
    }
    
    
    public function findOrganizationsAndTheirRespectiveBranches($organizationId)
    {
        $query = "SELECT ORG.NRORG, org.nrorgprincipal, LIC.CDLOCALUSO AS FILIAL, ORG.NMORGANIZACAO, ORG.NMRAZSOCIALORG
                    FROM ORGANIZACAO ORG
                    JOIN CLIENTE     CLI ON CLI.NRPARCNEGOCIO = ORG.NRPARCNEGOCIO
                    JOIN LICENCAUSO  LIC ON LIC.CDCLIENTE     = CLI.CDCLIENTE
                   WHERE LIC.IDLUSOATIV = 'S'
                     AND ORG.NRORG IN (SELECT ORG.NRORG
                                         FROM ORGANIZACAO ORG,
                                              (SELECT NVL(NRORGPRINCIPAL, NRORG) NRORG
                                                 FROM ORGANIZACAO
                                                WHERE NRORG = :NRORG) NRP
                                        WHERE ORG.NRORG          = NRP.NRORG
                                           OR ORG.NRORGPRINCIPAL = NRP.NRORG)
                                           
                ORDER BY ORG.NRORG";
        $params = array(
            'NRORG' => $organizationId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
    
    public function getOrganizationWithParcnegocio($nrParcNegocio)
    {
        $query = "SELECT NRORG FROM ORGANIZACAO WHERE NRPARCNEGOCIO = $nrParcNegocio";
       
        return $this->getEntityManager()->getConnection()->fetchAll($query)[0];
    }
    
    
    public function validateOperatorAlreadyCreated($nrorg, $dsemailoper)
    {
        $query = "SELECT DSEMAILOPER FROM ORGANIZACAO WHERE NRORGPRINCIPAL = :NRORG AND PREPSTATUS IN ('P', 'Z' ,'L' ,'Y', 'M', 'X') AND DSEMAILOPER LIKE (:DSEMAILOPER)";
       
        $params = array(
            'NRORG' => $nrorg,
            'DSEMAILOPER' => $dsemailoper
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
    
    
    
}